@@ -1,8 +1,6 @@
 //
 // Utility.cpp
 //
-// $Id: //poco/Main/Data/SQLite/src/Utility.cpp#5 $
-//
 // Library: Data/SQLite
 // Package: SQLite
 // Module:  Utility
@@ -61,6 +59,18 @@ Utility::TypeMap Utility::_types;
 Poco::Mutex Utility::_mutex;
 
 
+Utility::SQLiteMutex::SQLiteMutex(sqlite3* pDB): _pMutex(sqlite3_db_mutex(pDB))
+{
+	sqlite3_mutex_enter(_pMutex);
+}
+
+
+Utility::SQLiteMutex::~SQLiteMutex()
+{
+	sqlite3_mutex_leave(_pMutex);
+}
+
+
 Utility::Utility()
 {
 	if (_types.empty())
@@ -127,7 +137,11 @@ Utility::Utility()
 
 std::string Utility::lastError(sqlite3* pDB)
 {
-	return std::string(sqlite3_errmsg(pDB));
+	std::string errStr;
+	SQLiteMutex m(pDB);
+	const char* pErr = sqlite3_errmsg(pDB);
+	if (pErr) errStr = pErr;
+	return errStr;
 }
 
 
@@ -153,72 +167,74 @@ MetaColumn::ColumnDataType Utility::getColumnType(sqlite3_stmt* pStmt, std::size
 }
 
 
-void Utility::throwException(int rc, const std::string& addErrMsg)
+void Utility::throwException(sqlite3* pDB, int rc, const std::string& addErrMsg)
 {
 	switch (rc)
 	{
 	case SQLITE_OK:
 		break;
 	case SQLITE_ERROR:
-		throw InvalidSQLStatementException(std::string("SQL error or missing database"), addErrMsg);
+		throw InvalidSQLStatementException(lastError(pDB), addErrMsg);
 	case SQLITE_INTERNAL:
-		throw InternalDBErrorException(std::string("An internal logic error in SQLite"), addErrMsg);
+		throw InternalDBErrorException(lastError(pDB), addErrMsg);
 	case SQLITE_PERM:
-		throw DBAccessDeniedException(std::string("Access permission denied"), addErrMsg);
+		throw DBAccessDeniedException(lastError(pDB), addErrMsg);
 	case SQLITE_ABORT:
-		throw ExecutionAbortedException(std::string("Callback routine requested an abort"), addErrMsg);
+		throw ExecutionAbortedException(lastError(pDB), addErrMsg);
 	case SQLITE_BUSY:
 	case SQLITE_BUSY_RECOVERY:
+#if defined(SQLITE_BUSY_SNAPSHOT)
 	case SQLITE_BUSY_SNAPSHOT:
-		throw DBLockedException(std::string("The database file is locked"), addErrMsg);
+#endif
+		throw DBLockedException(lastError(pDB), addErrMsg);
 	case SQLITE_LOCKED:
-		throw TableLockedException(std::string("A table in the database is locked"), addErrMsg);
+		throw TableLockedException(lastError(pDB), addErrMsg);
 	case SQLITE_NOMEM:
-		throw NoMemoryException(std::string("A malloc() failed"), addErrMsg);
+		throw NoMemoryException(lastError(pDB), addErrMsg);
 	case SQLITE_READONLY:
-		throw ReadOnlyException(std::string("Attempt to write a readonly database"), addErrMsg);
+		throw ReadOnlyException(lastError(pDB), addErrMsg);
 	case SQLITE_INTERRUPT:
-		throw InterruptException(std::string("Operation terminated by sqlite_interrupt()"), addErrMsg);
+		throw InterruptException(lastError(pDB), addErrMsg);
 	case SQLITE_IOERR:
-		throw IOErrorException(std::string("Some kind of disk I/O error occurred"), addErrMsg);
+		throw IOErrorException(lastError(pDB), addErrMsg);
 	case SQLITE_CORRUPT:
-		throw CorruptImageException(std::string("The database disk image is malformed"), addErrMsg);
+		throw CorruptImageException(lastError(pDB), addErrMsg);
 	case SQLITE_NOTFOUND:
-		throw TableNotFoundException(std::string("Table or record not found"), addErrMsg);
+		throw TableNotFoundException(lastError(pDB), addErrMsg);
 	case SQLITE_FULL:
-		throw DatabaseFullException(std::string("Insertion failed because database is full"), addErrMsg);
+		throw DatabaseFullException(lastError(pDB), addErrMsg);
 	case SQLITE_CANTOPEN:
-		throw CantOpenDBFileException(std::string("Unable to open the database file"), addErrMsg);
+		throw CantOpenDBFileException(lastError(pDB), addErrMsg);
 	case SQLITE_PROTOCOL:
-		throw LockProtocolException(std::string("Database lock protocol error"), addErrMsg);
+		throw LockProtocolException(lastError(pDB), addErrMsg);
 	case SQLITE_EMPTY:
-		throw InternalDBErrorException(std::string("(Internal Only) Database table is empty"), addErrMsg);
+		throw InternalDBErrorException(lastError(pDB), addErrMsg);
 	case SQLITE_SCHEMA:
-		throw SchemaDiffersException(std::string("The database schema changed"), addErrMsg);
+		throw SchemaDiffersException(lastError(pDB), addErrMsg);
 	case SQLITE_TOOBIG:
-		throw RowTooBigException(std::string("Too much data for one row of a table"), addErrMsg);
+		throw RowTooBigException(lastError(pDB), addErrMsg);
 	case SQLITE_CONSTRAINT:
-		throw ConstraintViolationException(std::string("Abort due to constraint violation"), addErrMsg);
+		throw ConstraintViolationException(lastError(pDB), addErrMsg);
 	case SQLITE_MISMATCH:
-		throw DataTypeMismatchException(std::string("Data type mismatch"), addErrMsg);
+		throw DataTypeMismatchException(lastError(pDB), addErrMsg);
 	case SQLITE_MISUSE:
-		throw InvalidLibraryUseException(std::string("Library used incorrectly"), addErrMsg);
+		throw InvalidLibraryUseException(lastError(pDB), addErrMsg);
 	case SQLITE_NOLFS:
-		throw OSFeaturesMissingException(std::string("Uses OS features not supported on host"), addErrMsg);
+		throw OSFeaturesMissingException(lastError(pDB), addErrMsg);
 	case SQLITE_AUTH:
-		throw AuthorizationDeniedException(std::string("Authorization denied"), addErrMsg);
+		throw AuthorizationDeniedException(lastError(pDB), addErrMsg);
 	case SQLITE_FORMAT:
-		throw CorruptImageException(std::string("Auxiliary database format error"), addErrMsg);
+		throw CorruptImageException(lastError(pDB), addErrMsg);
 	case SQLITE_NOTADB:
-		throw CorruptImageException(std::string("File opened that is not a database file"), addErrMsg);
+		throw CorruptImageException(lastError(pDB), addErrMsg);
 	case SQLITE_RANGE:
-		throw InvalidSQLStatementException(std::string("Bind Parameter out of range (Access of invalid position 0? bind starts with 1!)"), addErrMsg);
+		throw InvalidSQLStatementException(lastError(pDB), addErrMsg);
 	case SQLITE_ROW:
 		break; // sqlite_step() has another row ready
 	case SQLITE_DONE:
 		break; // sqlite_step() has finished executing
 	default:
-		throw SQLiteException(std::string("Unknown error code: ") + Poco::NumberFormatter::format(rc), addErrMsg);
+		throw SQLiteException(Poco::format("Unknown error code: %d", rc), addErrMsg);
 	}
 }
 
