@@ -1,8 +1,6 @@
 //
 // File_WIN32U.cpp
 //
-// $Id: //poco/1.4/Foundation/src/File_WIN32U.cpp#1 $
-//
 // Library: Foundation
 // Package: Filesystem
 // Module:  File
@@ -63,7 +61,7 @@ FileImpl::FileImpl(const std::string& path): _path(path)
 	{
 		_path.resize(n - 1);
 	}
-	UnicodeConverter::toUTF16(_path, _upath);
+	convertPath(_path, _upath);
 }
 
 
@@ -87,7 +85,7 @@ void FileImpl::setPathImpl(const std::string& path)
 	{
 		_path.resize(n - 1);
 	}
-	UnicodeConverter::toUTF16(_path, _upath);
+	convertPath(_path, _upath);
 }
 
 
@@ -169,7 +167,12 @@ bool FileImpl::isDirectoryImpl() const
 
 bool FileImpl::isLinkImpl() const
 {
-	return false;
+	poco_assert (!_path.empty());
+
+	DWORD attr = GetFileAttributesW(_upath.c_str());
+	if (attr == INVALID_FILE_ATTRIBUTES)
+		handleLastErrorImpl(_path);
+	return (attr & FILE_ATTRIBUTE_DIRECTORY) == 0 && (attr & FILE_ATTRIBUTE_REPARSE_POINT) != 0;
 }
 
 
@@ -293,7 +296,7 @@ void FileImpl::copyToImpl(const std::string& path) const
 	poco_assert (!_path.empty());
 
 	std::wstring upath;
-	UnicodeConverter::toUTF16(path, upath);
+	convertPath(path, upath);
 	if (CopyFileW(_upath.c_str(), upath.c_str(), FALSE) == 0)
 		handleLastErrorImpl(_path);
 }
@@ -304,12 +307,41 @@ void FileImpl::renameToImpl(const std::string& path)
 	poco_assert (!_path.empty());
 
 	std::wstring upath;
-	UnicodeConverter::toUTF16(path, upath);
-	if (MoveFileW(_upath.c_str(), upath.c_str()) == 0)
+	convertPath(path, upath);
+	if (MoveFileExW(_upath.c_str(), upath.c_str(), MOVEFILE_REPLACE_EXISTING) == 0)
 		handleLastErrorImpl(_path);
 }
 
 
+void FileImpl::linkToImpl(const std::string& path, int type) const
+{
+	poco_assert (!_path.empty());
+
+	std::wstring upath;
+	convertPath(path, upath);
+
+	if (type == 0)
+	{
+		if (CreateHardLinkW(upath.c_str(), _upath.c_str(), NULL) == 0)
+			handleLastErrorImpl(_path);
+	}
+	else
+	{
+#if _WIN32_WINNT >= 0x0600 && defined(SYMBOLIC_LINK_FLAG_DIRECTORY)
+		DWORD flags = 0;
+		if (isDirectoryImpl()) flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
+#ifdef SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE
+		flags |= SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
+#endif
+		if (CreateSymbolicLinkW(upath.c_str(), _upath.c_str(), flags) == 0)
+			handleLastErrorImpl(_path);
+#else
+		throw Poco::NotImplementedException("Symbolic link support not available in used version of the Windows SDK");
+#endif
+	}
+}
+
+
 void FileImpl::removeImpl()
 {
 	poco_assert (!_path.empty());
@@ -384,7 +416,7 @@ void FileImpl::handleLastErrorImpl(const std::string& path)
 	case ERROR_CANNOT_MAKE:
 		throw CreateFileException(path, err);
 	case ERROR_DIR_NOT_EMPTY:
-		throw FileException("directory not empty", path, err);
+		throw DirectoryNotEmptyException(path, err);
 	case ERROR_WRITE_FAULT:
 		throw WriteFileException(path, err);
 	case ERROR_READ_FAULT:
@@ -406,4 +438,22 @@ void FileImpl::handleLastErrorImpl(const std::string& path)
 }
 
 
+void FileImpl::convertPath(const std::string& utf8Path, std::wstring& utf16Path)
+{
+	UnicodeConverter::toUTF16(utf8Path, utf16Path);
+	if (utf16Path.size() > MAX_PATH - 12) // Note: CreateDirectory has a limit of MAX_PATH - 12 (room for 8.3 file name)
+	{
+		if (utf16Path[0] == '\\' || utf16Path[1] == ':')
+		{
+			if (utf16Path.compare(0, 4, L"\\\\?\\", 4) != 0)
+			{
+				if (utf16Path[1] == '\\')
+					utf16Path.insert(0, L"\\\\?\\UNC\\", 8);
+				else
+					utf16Path.insert(0, L"\\\\?\\", 4);
+			}
+		}
+	}
+}
+
 } // namespace Poco
