@@ -1,8 +1,6 @@
 //
 // File_WIN32U.cpp
 //
-// $Id: //poco/1.4/Foundation/src/File_WINCE.cpp#1 $
-//
 // Library: Foundation
 // Package: Filesystem
 // Module:  File
@@ -301,6 +299,12 @@ void FileImpl::renameToImpl(const std::string& path)
 }
 
 
+void FileImpl::linkToImpl(const std::string& path, int type) const
+{
+	throw Poco::NotImplementedException("File::linkTo() is not available on this platform");
+}
+
+
 void FileImpl::removeImpl()
 {
 	poco_assert (!_path.empty());
